"""
Encapsulates properties useful for calling the OpenAI Assistants API.
"""
from typing import List, Optional, Union
from pydantic import BaseModel
from foundationallm.models.orchestration import (
    AnalysisResult,
    OpenAIFilePathMessageContentItem,
    OpenAIImageFileMessageContentItem,
    OpenAITextMessageContentItem
)

class OpenAIAssistantsAPIResponse(BaseModel):
    """
    Encapsulates properties after parsing the output of the OpenAI Assistants API.
       
    """
    document_id: Optional[str]
    content: Optional[
        List[
            Union[
                OpenAIFilePathMessageContentItem,
                OpenAIImageFileMessageContentItem,
                OpenAITextMessageContentItem
            ]
        ]
    ] = []
    analysis_results: Optional[List[AnalysisResult]] = []
    completion_tokens: Optional[int] = 0
    prompt_tokens: Optional[int] = 0
    total_tokens: Optional[int] = 0
    errors: Optional[List[str]] = []



"""
Base Agent fulfilling the orchestration request.
"""
from pydantic import Field
from typing import List, Optional, Union, Annotated
from foundationallm.models.agents import (
    AgentConversationHistorySettings,
    AgentGatekeeperSettings,
    AgentOrchestrationSettings,
    AzureAIAgentServiceAgentWorkflow,
    AzureOpenAIAssistantsAgentWorkflow,
    LangChainExpressionLanguageAgentWorkflow,
    LangGraphReactAgentWorkflow,
    ExternalAgentWorkflow,
    AgentTool
)
from foundationallm.models.resource_providers import ResourceBase

class AgentBase(ResourceBase):
    """ Agent Base model."""
    sessions_enabled: bool = Field(default=False, description="Indicates whether sessions are enabled for the agent.")
    conversation_history_settings: Optional[AgentConversationHistorySettings] = Field(default=AgentConversationHistorySettings(), description="Configuration for the agent's conversation history.")
    gatekeeper_settings: Optional[AgentGatekeeperSettings] = Field(default=AgentGatekeeperSettings(), description="Gatekeeper configuration for the agent.")
    orchestration_settings: Optional[AgentOrchestrationSettings] = Field(default=AgentOrchestrationSettings(), description="Agent settings for the orchestrator.")
    prompt_object_id: Optional[str] = Field(default=None, description="The object identifier of the Prompt object providing the prompt for the agent.")
    ai_model_object_id: Optional[str] = Field(default=None, description="The object identifier of the AIModelBase object providing the AI model for the agent.")
    capabilities:Optional[List[str]] = Field(default=[], description="The capabilities of the agent.")
    tools: Optional[List[AgentTool]] = Field(default=[], description="A list of assigned agent tools.")
    workflow: Optional[
        Annotated [
            Union[AzureAIAgentServiceAgentWorkflow, AzureOpenAIAssistantsAgentWorkflow, LangChainExpressionLanguageAgentWorkflow, LangGraphReactAgentWorkflow, ExternalAgentWorkflow],
            Field(discriminator='type')
        ]
    ]= Field(default=None, description="The workflow configuration for the agent.")



"""
Class: GatewayTextEmbeddingService
Description:  Class responsible for obtaining text embedding vectors from the Gateway API.
"""
import asyncio
import time
from foundationallm.config import Configuration, UserIdentity
from foundationallm.models.resource_providers.configuration import APIEndpointConfiguration
from foundationallm.models.services import GatewayTextEmbeddingResponse
from foundationallm.services import HttpClientService
from .text_chunk import TextChunk
from .text_embedding_request import TextEmbeddingRequest
from .text_embedding_response import TextEmbeddingResponse

class GatewayTextEmbeddingService():
    """
    Class for obtaining embedding vectors from the Gateway API.
    """
    def __init__(self,
                 instance_id:str,
                 user_identity:UserIdentity,
                 gateway_api_endpoint_configuration: APIEndpointConfiguration,
                 model_name:str,
                 config: Configuration,
                 model_dimensions:int = 1536):
        self.http_client = HttpClientService(gateway_api_endpoint_configuration, user_identity, config)
        self.model_name = model_name
        self.config = config
        self.url =  f'/instances/{instance_id}/embeddings'
        self.model_dimensions = model_dimensions

    def get_embedding(self, text: str) -> GatewayTextEmbeddingResponse:
        """
        Get the embedding vector for a given text.
        """

        if (text == None) or (len(text) == 0):
            raise Exception("Text is empty or None")

        # create the text embedding request
        text_embedding_request = self._create_text_embedding_request(text)

        # start the operation
        request_json = text_embedding_request.model_dump_json(by_alias=True)

        resp = self.http_client.post(self.url, data=request_json)
        response = TextEmbeddingResponse.model_validate(resp)
        # poll until completion
        while response.in_progress and not response.failed:
            # delay for a second in between polling
            time.sleep(1)
            get_resp = self.http_client.get(self.url + f'?operationId={response.operation_id}')
            response = TextEmbeddingResponse.model_validate(get_resp)

        if response.failed:
            raise Exception(f"Text embedding operation failed: {response.error_message}")

        return GatewayTextEmbeddingResponse(
            embedding_vector=response.text_chunks[0].embedding,
            tokens_count=response.text_chunks[0].tokens_count)

    async def aget_embedding(self, text: str) -> GatewayTextEmbeddingResponse:
        """
        Asynchronously get the embedding vector for a given text.
        """

        text_embedding_request = self._create_text_embedding_request(text)

        # start the operation
        request_json = text_embedding_request.model_dump_json(by_alias=True)

        # Send asynchronous POST request to start the operation
        response = TextEmbeddingResponse.model_validate(await self.http_client.post_async(self.url, data=request_json))

        # Poll until operation is complete
        while response.in_progress and not response.failed:
            await asyncio.sleep(1)  # Use asyncio.sleep for non-blocking delay
            response = TextEmbeddingResponse.model_validate(await self.http_client.get_async(self.url + f'?operationId={response.operation_id}'))

        if response.failed:
            raise Exception(f"Text embedding operation failed: {response.error_message}")

        return GatewayTextEmbeddingResponse(
            embedding_vector=response.text_chunks[0].embedding,
            tokens_count=response.text_chunks[0].tokens_count)

    def _create_text_embedding_request(self, text: str) -> TextEmbeddingRequest:
        text_chunk = TextChunk(content=text)
        return TextEmbeddingRequest(text_chunks=[text_chunk], embedding_model_name=self.model_name, embedding_model_dimensions=self.model_dimensions, prioritized=True)



class RunnableConfigKeys:
    """The keys for RunnableConfig values."""
    ORIGINAL_USER_PROMPT = 'original_user_prompt'
    ORIGINAL_USER_PROMPT_REWRITE = 'original_user_prompt_rewrite'
    CONVERSATION_ID = 'conversation_id'
    FILE_HISTORY = 'file_history'



from typing import Optional
from langchain_core.documents import Document

class VectorDocument(Document):

    id : str
    score: float
    rerank_score: Optional[float]
    
    class Config:
        extra = "allow"



from .content_artifact_retrieval_base import ContentArtifactRetrievalBase
from .azure_ai_search_service_retriever import AzureAISearchServiceRetriever
from .search_service_filter_retriever import SearchServiceFilterRetriever
from .retriever_factory import RetrieverFactory
